import argparse
import subprocess

import zcommons as zc


def main(args):

    @zc.timer(name=args.cmds[0], repeat=args.repeat, show=True)
    def run():
        subprocess.run(args.cmds, shell=False)

    run()


def parse_args():
    parser = argparse.ArgumentParser(description="timer")
    parser.add_argument("--repeat", type=int, default=1, help="exec times")
    parser.add_argument("cmds", metavar="N", type=str, nargs="*", help="exec commands")
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    main(args)



__all__ = [
    "Timer",
    "timer"
]

from functools import wraps
from typing import Union, List, Dict, Tuple

from .units import TimeUnits
from . import time as time


class Timer(object):
    """
    A Single Thread Timer
    =====================
    A timer which supports start, pause, record, etc.
    """

    def __init__(self, name: str = "Timer", unit: Union[str, TimeUnits] = TimeUnits.SECOND):
        """

        :param name: a meaningful string representing the timer function.
        :param unit: the unit of result.
        """
        super(Timer, self).__init__()
        self.name = name
        self.unit = TimeUnits(unit)
        self.__records = []
        self.__elapsed_time_ns = [0, 0]
        self.__record_time_ns = [0, 0]
        self.__start_time_point_ns = [0, 0]
        self.__stop_time_point_ns = [0, 0]
        self.__running = False
        self.__pausing = False

    def start(self) -> None:
        """
        Start the timer.
        There will have no effect if the timer already started.

        :return:
        """
        if self.__running:
            return
        self.__running = True
        self.__start_time_point_ns = time.perf_counter_ns(), time.process_time_ns()

    def pause(self) -> None:
        """
        Pause the timer.
        There will have no effect if the timer not in running or already paused.

        :return:
        """
        if not self.__running or self.__pausing:
            return
        self.__pausing = True
        pc, pt = time.perf_counter_ns(), time.process_time_ns()
        self.__record_time_ns[0] += pc - self.__start_time_point_ns[0]
        self.__record_time_ns[1] += pt - self.__start_time_point_ns[1]

    def resume(self) -> None:
        """
        Resume the timer from pausing status.
        There will have no effect if the timer not in running or not in pausing.

        :return:
        """
        if not self.__running or not self.__pausing:
            return
        self.__start_time_point_ns = [time.perf_counter_ns(), time.process_time_ns()]
        self.__pausing = False

    def record(self) -> None:
        """
        Generate a new time record.
        There will have no effect if the timer not in running.

        :return:
        """
        self.__record(is_stop=False)

    def stop(self) -> None:
        """
        Stop the timer and generate a new time record.
        There will have no effect if the timer not in running.

        :return:
        """
        self.__record(is_stop=True)
        self.__running = False

    def elapsed_records(self, unit: Union[str, TimeUnits] = None, exclude_zero: bool = False) -> List[Dict[str, int]]:
        """
        Get all records with specified unit.

        :param unit: the time unit of result, the ``self.unit`` will be used if it is ``None``.
        :param exclude_zero:  whether ignore the records which have zero time duration. If call ``record`` repeatly in
        pausing status, zero time duration records will be generated.
        :return: a record list.
        """
        if unit is None:
            unit = self.unit
        ret = []
        for r in self.__records:
            if exclude_zero and r[0] == 0 and r[1] == 0:
                continue
            ret.append({
                "perf_counter": TimeUnits.NANO.convert_to(r[0], unit),
                "process_time": TimeUnits.NANO.convert_to(r[1], unit)
            })
        return ret

    def last_record(self, unit: Union[str, TimeUnits] = None, exclude_zero: bool = False) -> Tuple[int, int]:
        """
        The elapsed time from last record with ``unit``.

        :param unit: the time unit of result, the ``self.unit`` will be used if it is ``None``.
        :param exclude_zero: whether ignore the records which have zero time duration.
        :return:
        """
        if unit is None:
            unit = self.unit
        pc_ns, pt_ns = self.last_record_ns(exclude_zero)
        pc = TimeUnits.NANO.convert_to(pc_ns, unit)
        pt = TimeUnits.NANO.convert_to(pt_ns, unit)
        return pc, pt

    def last_record_ns(self, exclude_zero: bool = False) -> Tuple[int, int]:
        """
        The elapsed nanoseconds from last record.

        :param exclude_zero: whether ignore the records which have zero time duration.
        :return:
        """
        if not self.__running:
            for i in range(len(self.__records) - 1, -1, -1):
                if not exclude_zero or (self.__records[i][0] == 0 and self.__records[i][1] == 0):
                    return tuple(self.__records[i])
            return 0, 0
        else:
            if self.__pausing:
                return self.__record_time_ns[0], self.__record_time_ns[1]
            else:
                pc, pt = time.perf_counter_ns(), time.process_time_ns()
                return self.__record_time_ns[0] + pc - self.__start_time_point_ns[0], \
                       self.__record_time_ns[1] + pt - self.__start_time_point_ns[1]

    def elapsed(self, unit: Union[str, TimeUnits] = None) -> Tuple[int, int]:
        """
        The elapsed time from started time point with ``unit``.
        If the timer already stopped, this method will return the duration from started to stopped time point.

        :param unit: the time unit, the ``self.unit`` will be used if it is ``None``.
        :return: a tuple of perf counter and process time
        """
        if unit is None:
            unit = self.unit
        pc_ns, pt_ns = self.elapsed_ns()
        pc = TimeUnits.NANO.convert_to(pc_ns, unit)
        pt = TimeUnits.NANO.convert_to(pt_ns, unit)
        return pc, pt

    def elapsed_ns(self) -> Tuple[int, int]:
        """
        The elapsed nanoseconds from started time point.
        If the timer already stopped, this method will return the duration from started to stopped time point.

        :return: a tuple of perf counter and process time
        """
        if not self.__running:
            return self.__elapsed_time_ns[0], self.__elapsed_time_ns[1]
        else:
            rec_pc, rec_pt = self.__record_elapsed()
            return self.__elapsed_time_ns[0] + rec_pc, self.__elapsed_time_ns[1] + rec_pt

    def __record(self, is_stop=False):
        if not self.__running:
            return
        rec_pc, rec_pt = self.__record_elapsed(is_stop=is_stop)
        self.__records.append((rec_pc, rec_pt))
        self.__record_time_ns = [0, 0]
        self.__elapsed_time_ns[0] += rec_pc
        self.__elapsed_time_ns[1] += rec_pt

    def __record_elapsed(self, is_stop=False):
        rec_pc, rec_pt = self.__record_time_ns
        pc, pt = time.perf_counter_ns(), time.process_time_ns()
        if not self.__pausing:
            rec_pc += pc - self.__start_time_point_ns[0]
            rec_pt += pt - self.__start_time_point_ns[1]
        self.__start_time_point_ns = [pc, pt]
        if is_stop:
            self.__stop_time_point_ns = [pc, pt]
        return rec_pc, rec_pt

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()


def __show_timer_res(res: dict):
    print(f"Run {res['name']} for {res['repeat']} times.")
    print(f"  |- Total perf counter: {res['perf_counter']}{res['unit']}")
    print(f"  |- Total process time: {res['process_time']}{res['unit']}")
    print(f"  |- Avg   perf counter: {res['perf_counter'] / res['repeat']}{res['unit']}")
    print(f"  |- Avg   process time: {res['process_time'] / res['repeat']}{res['unit']}")
    print()
    print(f"The following is a detailed time record of each run:")

    def __left_pad_zero(x: int, ndigits: int):
        x_str = str(x)
        zero_str = "0" * (ndigits - len(x_str))
        return zero_str + x_str

    if res['repeat'] <= 10:
        for i in range(res["repeat"]):
            print(f"Run {i + 1:02} - perf counter: {res['records'][i]['perf_counter']}{res['unit']}")
            print(f"       - process time: {res['records'][i]['process_time']}{res['unit']}")
    else:
        ndigits = len(str(res["repeat"]))
        for i in range(5):
            print(
                f"Run {__left_pad_zero(i + 1, ndigits)} - perf counter: {res['records'][i]['perf_counter']}{res['unit']}")
            print(f"    {' ' * ndigits} - process time: {res['records'][i]['process_time']}{res['unit']}")
        print(f"....{'.' * ndigits}")
        for i in range(res["repeat"] - 5, res["repeat"]):
            print(
                f"Run {__left_pad_zero(i + 1, ndigits)} - perf counter: {res['records'][i]['perf_counter']}{res['unit']}")
            print(f"    {' ' * ndigits} - process time: {res['records'][i]['process_time']}{res['unit']}")


def timer(name: str = None, unit: Union[str, TimeUnits] = TimeUnits.SECOND, repeat: int = 1, res: dict = None,
          show: bool = False):
    """
    the decorator of Timer.

    :param name: the timer task name
    :param unit: the unit of res
    :param repeat: exec task function times
    :param res: a dict for storing result
    :param show: whether print timer info to stdout
    :return: the first exec result of task func
    """

    def decorate(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            real_name = name
            if real_name is None:
                real_name = getattr(func, "__name__", None)
            if real_name is None:
                real_name = "timer"
            real_res = res
            if real_res is None:
                real_res = {}

            with Timer(real_name, unit) as tmer:
                ret = func(*args, **kwargs)
                if repeat > 1:
                    for _ in range(repeat - 1):
                        tmer.record()
                        func(*args, **kwargs)

            pc, pt = tmer.elapsed()
            real_res["name"] = real_name
            real_res["unit"] = TimeUnits(unit).value
            real_res["repeat"] = repeat
            real_res["perf_counter"] = pc
            real_res["process_time"] = pt
            real_res["records"] = tmer.elapsed_records()
            if show:
                __show_timer_res(real_res)

            return ret

        return wrapper

    return decorate



__all__ = [
    "Counter",
    "LockFreeCounter",
    "MultiThreadCounter",
    "MultiProcessCounter"
]

import itertools
import threading
import multiprocessing


class Counter(object):
    """
    Basic Counter
    =============
    The basic and fastest counter, non-thread-safe, only can be used in single thread.
    """

    def __init__(self, v: int = 0, step: int = 1):
        super(Counter, self).__init__()
        self._v = v
        self._step = step

    def inc(self) -> None:
        """
        let ``count`` increase ``step``

        :return:
        """
        self._v += self._step

    def dec(self) -> None:
        """
        let ``count`` decrease ``step``

        :return:
        """
        self._v -= self._step

    def count(self) -> int:
        """
        the current value of counter

        :return: an int value.
        """
        return self._v


class LockFreeCounter(Counter):

    """
    Lock-Free Counter
    =================
    The thread-safe counter in CPython.
    Due to the GIL, CPython can implement lock-free counter
    """

    def __init__(self, v: int = 0, step: int = 1):
        super(LockFreeCounter, self).__init__(v, step)
        self._r_cnt = itertools.count(v, step)
        self._l_cnt = itertools.count(0, step)

    def inc(self) -> None:
        next(self._r_cnt)

    def dec(self) -> None:
        next(self._l_cnt)

    def count(self) -> int:
        return next(self._r_cnt) - next(self._l_cnt)


class MultiThreadCounter(Counter):

    """
    Multi-Thread Counter
    ====================
    The thread-safe counter for every python interpreter.
    Due to this implementation uses lock, it is slower than ``LockFreeCounter``.
    """

    def __init__(self, v: int = 0, step: int = 1):
        super(MultiThreadCounter, self).__init__(v, step)
        self._lock = threading.Lock()

    def inc(self) -> None:
        with self._lock:
            self._v += self._step

    def dec(self) -> None:
        with self._lock:
            self._v -= self._step

    def count(self) -> int:
        return self._v


class MultiProcessCounter(Counter):

    """
    Multi-Process Counter
    =====================
    The process-safe counter.
    Due to process sync, this counter is the slowest counter.
    """

    def __init__(self, v: int = 0, step: int = 1):
        super(MultiProcessCounter, self).__init__(v, step)
        self._v = multiprocessing.Value("l", v)
        self._step = multiprocessing.Value("l", step)

    def inc(self) -> None:
        with self._v.get_lock():
            self._v.value = self._v.value + self._step.value

    def dec(self) -> None:
        with self._v.get_lock():
            self._v.value = self._v.value - self._step.value

    def count(self) -> int:
        return self._v.value



"""
Colorama extension
"""

__all__ = [
    "cprint",
    "init", "deinit", "reinit", "colorama_text",
    "Fore", "Back", "Style", "Cursor",
    "AnsiToWin32"
]

import sys

from colorama import *
from colorama.ansi import code_to_chars


def __extend_colorama():
    def __add_code(cls, name, code):
        setattr(cls, name, code_to_chars(code))

    __add_code(Style, "ITALIC", 3)
    __add_code(Style, "UNDERLINE", 4)
    __add_code(Style, "SLOW_BLINK", 5)
    __add_code(Style, "RAPID_BLINK", 6)
    __add_code(Style, "INVERT", 7)
    __add_code(Style, "HIDE", 8)
    __add_code(Style, "STRIKE", 9)
    __add_code(Style, "DOUBLE_UNDERLINE", 21)


__extend_colorama()
init()


def __wrap_value(v, style):
    style = "".join(style)
    return style + v + Style.RESET_ALL


def cprint(*values, sep=' ', end='\n', file=sys.stdout, flush=False, style=None) -> None:
    """
    Print text in the terminal with styles such as color.

    :param values:
    :param sep:
    :param end:
    :param file:
    :param flush:
    :param style: the text styles, str or list.
    :return:
    """
    if style is None:
        print(*values, sep=sep, end=end, file=file, flush=flush)
    else:
        if file is sys.__stdout__ or file is sys.__stderr__ or (hasattr(file, "isatty") and file.isatty()):
            new_values = [__wrap_value(v, style) for v in values]
            print(*new_values, sep=sep, end=end, file=file, flush=flush)
        else:
            print(*values, sep=sep, end=end, file=file, flush=flush)



import importlib
import os
import sys


def add_sys_path(path):
    sys.path.insert(0, path)


def del_sys_path(path, max=-1, cmp=None):
    if not cmp:
        cmp = lambda l, r: l == r
    tmp = []
    cnt = 0
    for p in sys.path:
        if 0 <= max <= cnt:
            break
        if not cmp(p, path):
            tmp.append(p)
            cnt += 1
    sys.path = tmp


class TmpSysPath(object):

    def __init__(self, path=None):
        super(TmpSysPath, self).__init__()
        self.__path = os.path.abspath(path) if path is not None else None

    def __enter__(self):
        if self.__path is not None:
            add_sys_path(self.__path)

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.__path is not None:
            del_sys_path(self.__path)


def import_module(name, package=None, path=None):
    with TmpSysPath(path):
        return importlib.import_module(name, package)


def _resolve_name(name, package, level):
    import importlib._bootstrap as bs
    bs._sanity_check(name, package, level)
    if level > 0:
        return bs._resolve_name(name, package, level)
    else:
        return name


def _import_object(name):
    tokens = name.split(".")
    is_module = True
    mod = importlib.import_module(tokens[0])
    for tk in tokens[1:]:
        if is_module:
            try:
                mod = importlib.import_module(f".{tk}", mod.__name__)
            except:
                is_module = False
        if not is_module:
            mod = getattr(mod, tk)
    return mod


def import_object(name, package=None, path=None):
    level = 0
    if name.startswith('.'):
        if not package:
            msg = ("the 'package' argument is required to perform a relative "
                   "import for {!r}")
            raise TypeError(msg.format(name))
        for character in name:
            if character != '.':
                break
            level += 1
    name = _resolve_name(name[level:], package, level)
    with TmpSysPath(path):
        return _import_object(name)




class Config(object):

    def __init__(self, data: dict):
        super(Config, self).__init__()
        self.__data = data

    def get(self, key, default=None):
        k_seq = self.__split_key(key)
        return self.__get_from_dict(self.__data, k_seq, default)

    def set(self, key, value):
        k_seq = self.__split_key(key)
        self.__set_to_dict(self.__data, k_seq, value)

    def remove(self, key):
        k_seq = self.__split_key(key)
        self.__remove_from_dict(self.__data, k_seq)

    @classmethod
    def __split_key(cls, key: str):
        if key is None or key.strip() == "":
            raise ValueError("key cannot be none or empty.")
        return key.split(".")

    @classmethod
    def __exists_in_dict(cls, d: dict, k_seq: list):
        if k_seq is None or len(k_seq) == 0:
            return False
        for k in k_seq:
            if k in d:
                d = d[k]
            else:
                return False
        return True

    @classmethod
    def __get_from_dict(cls, d: dict, k_seq: list, default=None):
        if k_seq is None or len(k_seq) == 0:
            raise ValueError("key cannot be none or empty")
        for k in k_seq:
            if k in d:
                d = d[k]
            else:
                return False, default
        return True, d

    @classmethod
    def __remove_from_dict(cls, d: dict, k_seq: list):
        if k_seq is None or len(k_seq) == 0:
            raise ValueError("key cannot be none or empty")
        for k in k_seq[:-1]:
            if k not in d:
                return False
            d = d[k]
        try:
            del d[k_seq[-1]]
            return True
        except:
            return False

    @classmethod
    def __set_to_dict(cls, d: dict, k_seq: list, value):
        if k_seq is None or len(k_seq) == 0:
            raise ValueError("key cannot be none or empty")
        for k in k_seq[:-1]:
            if k not in d:
                d[k] = {}
            d = d[k]
        d[k_seq[-1]] = value



__all__ = [
    "ThreadGroup"
]

import threading

from .counter import MultiThreadCounter


class ThreadGroup(object):

    """
    Thread Group
    ============
    A simple thread group
    """

    def __init__(self):
        super(ThreadGroup, self).__init__()
        self.threads = []
        self.__counter = MultiThreadCounter()

    def __run(self, target, *args, **kwargs):
        self.__counter.inc()
        target(*args, **kwargs)
        self.__counter.dec()

    def add(self, target=None, name=None, args=(), kwargs=None, *, daemon=None) -> threading.Thread:
        """
        Add a thread, the meaning of the parameters is the same as ``threading.Thread`` class.

        :param target:
        :param name:
        :param args:
        :param kwargs:
        :param daemon:
        :return:
        """
        real_args = (target, *args)
        t = threading.Thread(target=self.__run, name=name, args=real_args, kwargs=kwargs, daemon=daemon)
        self.threads.append(t)
        return t

    def start(self) -> None:
        """
        Start all threads in group

        :return:
        """
        for t in self.threads:
            t.start()

    def join(self) -> None:
        """
        Wait until the thread group terminates.

        :return:
        """
        for t in self.threads:
            t.join()

    def done(self) -> bool:
        """
        Whether the all the threads are done.

        :return:
        """
        return self.__counter.count() == 0

    def alive_count(self) -> int:
        """
        The count of running threads.

        :return:
        """
        return self.__counter.count()

    def count(self) -> int:
        """
        The count of all threads.

        :return:
        """
        return len(self)

    def __len__(self):
        return len(self.threads)

    def __getitem__(self, item):
        return self.threads[item]



__all__ = [
    "asdict",
    "asobj"
]

import dataclasses
import sys
import typing

_BUILTIN_BASE_TYPES = [type(None), bool, int, float, str, bytes]
_BUILTIN_CONTAIN_TYPES = [tuple, list, set, dict]


def asdict(obj, *, dict_factory=dict):
    """
    Convert object to dict.
    If the type of ``object`` is built-in types, the type of return value is same with the type of ``object``.

    :param obj: the object of built-in types or custom class with ``dataclass`` decorator.
    :param dict_factory:  the real type of return dict.
    :return:
    """
    if dataclasses.is_dataclass(obj):
        return dataclasses.asdict(obj, dict_factory=dict_factory)
    # return itself directly if obj is base type
    if type(obj) in _BUILTIN_BASE_TYPES:
        return obj
    # traverse container and check the sameness of elements in container
    if type(obj) in _BUILTIN_CONTAIN_TYPES:
        key_type, val_type = None, None
        if type(obj) is dict:
            ret = dict_factory()
            for k, v in obj.items():
                if key_type is None:
                    key_type, val_type = type(k), type(v)
                else:
                    if key_type != type(k) or val_type != type(v):
                        raise TypeError(f"asdict() should be called on container with the same element type")

                ret[asdict(k)] = asdict(v)
            return ret
        else:
            ret = []
            for k in obj:
                if key_type is None:
                    key_type = type(k)
                else:
                    if key_type != type(k):
                        raise TypeError(f"asdict() should be called on container with the same element type")

                ret.append(asdict(k))
            return type(obj)(ret)


def asobj(_cls, d):
    """
    Construct object from dict.

    :param _cls: the type of return value.
    :param d: the data dict.
    :return: a object of ``_cls``
    """
    if d is None:
        return None
    if not isinstance(_cls, type) and not _is_typing(_cls):
        raise TypeError(f"asobj() should be called on type")
    if dataclasses.is_dataclass(_cls):
        init_params = {}
        set_attrs = {}

        for f in getattr(_cls, dataclasses._FIELDS).values():
            # Only consider normal fields
            if f._field_type in [dataclasses._FIELD_CLASSVAR, dataclasses._FIELD_INITVAR]:
                continue

            v = None
            if f.name in d:
                v = asobj(f.type, d[f.name])
            else:
                if f.default is dataclasses._MISSING_TYPE and f.default_factory is dataclasses._MISSING_TYPE:
                    raise ValueError(f"Field {f.name} is required, but its value is missing")

            if f.init:
                init_params[f.name] = v
            else:
                set_attrs[f.name] = v

        ret = _cls(**init_params)
        for k, v in set_attrs.items():
            setattr(ret, k, v)
        return ret

    if _is_typing(_cls):
        if _cls is typing.Any:
            return d
        if _cls.__dict__["__args__"] is None:
            raise TypeError(f"asobj() should be called on container which specified element type")
        orig_base = _origin(_cls)
        if orig_base is dict:
            key_type, val_type = _cls.__dict__["__args__"]
            if type(d) is not dict:
                raise TypeError(f"the type of d is not {_cls}")
            ret = dict()
            for k, v in d.items():
                ret[asobj(key_type, k)] = asobj(val_type, v)
            return ret
        if orig_base in [tuple, list, set]:
            key_type, = _cls.__dict__["__args__"]
            ret = []
            # let it raises
            for k in d:
                ret.append(asobj(key_type, k))
            return orig_base(ret)

    if _cls in _BUILTIN_BASE_TYPES:
        return _cls(d)

    if _cls in _BUILTIN_CONTAIN_TYPES:
        raise TypeError(f"asobj() should be called on container which specified element type")
    raise TypeError(f"unsupport type {_cls}")


def __py36_is_typing(_cls):
    from typing import GenericMeta
    return isinstance(_cls, GenericMeta)


def __py37_is_typing(_cls):
    from typing import _Final
    return isinstance(_cls, _Final)


def __py36_origin(_cls):
    return _cls.__dict__["__extra__"]


def __py37_origin(_cls):
    return _cls.__dict__["__origin__"]


if sys.version_info.minor > 6:
    _is_typing = __py37_is_typing
    _origin = __py37_origin
else:
    _is_typing = __py36_is_typing
    _origin = __py36_origin


def _is_typing_container(_cls):
    return _is_typing(_cls) and _origin(_cls) in _BUILTIN_CONTAIN_TYPES



"""
Units
=====
Some common units such as time, number and bytes.
Every unit class is an enum, and has two convert methods, named ``convert_to`` and ``convert_from``.
"""

__all__ = [
    "TimeUnits",
    "NumberUnits",
    "BinaryUnits"
]

from enum import Enum
from typing import Union


_convert_values = {}


class _Converter:
    
    def convert_from(self, number: Union[int, float], src_unit) -> Union[int, float]:
        """
        convert ``number`` from ``src_unit`` to ``self``.

        :param number: a number with int or float type
        :param src_unit: the unit of ``number``
        :return: the return value has the same type with ``number``, it means, if the type of ``number`` is int, the
        method will use integer division.
        """
        return self.__convert(number, src_unit, self)

    def convert_to(self, number: Union[int, float], dst_unit) -> Union[int, float]:
        """
        convert ``number`` from ``self`` to ``dst_unit``.

        :param number: a number with int or float type
        :param dst_unit: the unit of return value
        :return: the return value has the same type with ``number``.
        """
        return self.__convert(number, self, dst_unit)

    @classmethod
    def __convert(cls, number, src_unit, dst_unit):
        if isinstance(src_unit, str):
            src_unit = cls(src_unit)
        if isinstance(dst_unit, str):
            dst_unit = cls(dst_unit)
        if type(src_unit) != type(dst_unit):
            raise TypeError(f"the two units must be equal, but got {type(src_unit)} and {type(dst_unit)}")
        enum_cls = type(src_unit)
        values = _convert_values[enum_cls]
        if isinstance(number, int):
            if values[src_unit] > values[dst_unit]:
                scale = values[src_unit] // values[dst_unit]
                return number * scale
            else:
                scale = values[dst_unit] // values[src_unit]
                return number // scale
        elif isinstance(number, float):
            if values[src_unit] > values[dst_unit]:
                scale = values[src_unit] // values[dst_unit]
                return number * scale
            else:
                scale = values[dst_unit] // values[src_unit]
                return number / scale
        else:
            raise TypeError(f"the type of number must be int or float, not {type(number)}")


class TimeUnits(_Converter, Enum):
    """
    Time Units
    ==========
    The time units, from nanoseconds up to hours.
    """
    NANO = "ns"
    MICRO = "us"
    MILLI = "ms"
    SECOND = "s"
    MINUTE = "m"
    HOUR = "h"


_time_values = {
    TimeUnits.NANO: 1,
    TimeUnits.MICRO: 1000,
    TimeUnits.MILLI: 1000000,
    TimeUnits.SECOND: 1000000000,
    TimeUnits.MINUTE: 60000000000,
    TimeUnits.HOUR: 3600000000000
}
_convert_values[TimeUnits] = _time_values


class NumberUnits(_Converter, Enum):
    """
    Number Units
    ============
    The decimal number unit, from one up to peta(1e15).
    """
    BASE = "b"
    DECA = "d"
    HECTO = "h"
    KILO = "k"
    MEGA = "m"
    GIGA = "g"
    TERA = "t"
    PETA = "p"
    
    
_number_values = {
    NumberUnits.BASE: 1,
    NumberUnits.DECA: 10,
    NumberUnits.HECTO: 100,
    NumberUnits.KILO: 1000,
    NumberUnits.MEGA: 1000000,
    NumberUnits.GIGA: 1000000000,
    NumberUnits.TERA: 1000000000000,
    NumberUnits.PETA: 1000000000000000
}
_convert_values[NumberUnits] = _number_values


class BinaryUnits(_Converter, Enum):
    """
    Binary Units
    ============
    The bit units and byte units, from B(b) up to PiB(b).
    """
    
    b = "b"
    B = "B"
    Kib = "Kib"
    KiB = "KiB"
    Mib = "Mib"
    MiB = "MiB"
    Gib = "Gib"
    GiB = "GiB"
    Tib = "Tib"
    TiB = "TiB"
    Pib = "Pib"
    PiB = "PiB"
    

_binary_values = {
    BinaryUnits.b: 1,
    BinaryUnits.B: 8,
    BinaryUnits.Kib: 1024,
    BinaryUnits.KiB: 8192,
    BinaryUnits.Mib: 1048576,
    BinaryUnits.MiB: 8388608,
    BinaryUnits.Gib: 1073741824,
    BinaryUnits.GiB: 8589934592,
    BinaryUnits.Tib: 1099511627776,
    BinaryUnits.TiB: 8796093022208,
    BinaryUnits.Pib: 1125899906842624,
    BinaryUnits.PiB: 9007199254740992
}
_convert_values[BinaryUnits] = _binary_values



import setuptools


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setuptools.setup(
    name="zcommons",
    version="0.1.0b1",
    author="malanore.z",
    author_email="malanore.z@gmail.com",
    description="A collection of common utils for python.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/malanore-z/zcommons4p",
    project_urls={
        "Bug Tracker": "https://github.com/malanore-z/zcommons4p/issues",
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    packages=setuptools.find_packages(".", include=("zcommons", "zcommons.*")),
    package_data={
        "zcommons": ["resources/*"]
    },
    include_package_data=True,
    install_requires=[
        "colorama",
        "dataclasses;python_version<'3.7'"
    ]
)



import zcommons.o2d
import zcommons.module
import zcommons.units
from zcommons.colorama import cprint
from zcommons.config import Config
from zcommons.counter import *
from zcommons.threadgroup import *
from zcommons.timer import *

# alias for colorama colors
FORE_BLACK = zcommons.colorama.Fore.BLACK
FORE_BLUE = zcommons.colorama.Fore.BLUE
FORE_CYAN = zcommons.colorama.Fore.CYAN
FORE_GREEN = zcommons.colorama.Fore.GREEN
FORE_MAGENTA = zcommons.colorama.Fore.MAGENTA
FORE_RED = zcommons.colorama.Fore.RED
FORE_RESET = zcommons.colorama.Fore.RESET
FORE_WHITE = zcommons.colorama.Fore.WHITE
FORE_YELLOW = zcommons.colorama.Fore.YELLOW
FORE_LIGHTBLACK_EX = zcommons.colorama.Fore.LIGHTBLACK_EX
FORE_LIGHTBLUE_EX = zcommons.colorama.Fore.LIGHTBLUE_EX
FORE_LIGHTCYAN_EX = zcommons.colorama.Fore.LIGHTCYAN_EX
FORE_LIGHTGREEN_EX = zcommons.colorama.Fore.LIGHTGREEN_EX
FORE_LIGHTMAGENTA_EX = zcommons.colorama.Fore.LIGHTMAGENTA_EX
FORE_LIGHTRED_EX = zcommons.colorama.Fore.LIGHTRED_EX
FORE_LIGHTWHITE_EX = zcommons.colorama.Fore.LIGHTWHITE_EX
FORE_LIGHTYELLOW_EX = zcommons.colorama.Fore.LIGHTYELLOW_EX

BACK_BLACK = zcommons.colorama.Back.BLACK
BACK_BLUE = zcommons.colorama.Back.BLUE
BACK_CYAN = zcommons.colorama.Back.CYAN
BACK_GREEN = zcommons.colorama.Back.GREEN
BACK_MAGENTA = zcommons.colorama.Back.MAGENTA
BACK_RED = zcommons.colorama.Back.RED
BACK_RESET = zcommons.colorama.Back.RESET
BACK_WHITE = zcommons.colorama.Back.WHITE
BACK_YELLOW = zcommons.colorama.Back.YELLOW
BACK_LIGHTBLACK_EX = zcommons.colorama.Back.LIGHTBLACK_EX
BACK_LIGHTBLUE_EX = zcommons.colorama.Back.LIGHTBLUE_EX
BACK_LIGHTCYAN_EX = zcommons.colorama.Back.LIGHTCYAN_EX
BACK_LIGHTGREEN_EX = zcommons.colorama.Back.LIGHTGREEN_EX
BACK_LIGHTMAGENTA_EX = zcommons.colorama.Back.LIGHTMAGENTA_EX
BACK_LIGHTRED_EX = zcommons.colorama.Back.LIGHTRED_EX
BACK_LIGHTWHITE_EX = zcommons.colorama.Back.LIGHTWHITE_EX
BACK_LIGHTYELLOW_EX = zcommons.colorama.Back.LIGHTYELLOW_EX

STYLE_BRIGHT = zcommons.colorama.Style.BRIGHT
STYLE_DIM = zcommons.colorama.Style.DIM
STYLE_NORMAL = zcommons.colorama.Style.NORMAL
STYLE_RESET_ALL = zcommons.colorama.Style.RESET_ALL
STYLE_ITALIC = zcommons.colorama.Style.ITALIC
STYLE_UNDERLINE = zcommons.colorama.Style.UNDERLINE
STYLE_SLOW_BLINK = zcommons.colorama.Style.SLOW_BLINK
STYLE_RAPID_BLINK = zcommons.colorama.Style.RAPID_BLINK
STYLE_INVERT = zcommons.colorama.Style.INVERT
STYLE_HIDE = zcommons.colorama.Style.HIDE
STYLE_STRIKE = zcommons.colorama.Style.STRIKE
STYLE_DOUBLE_UNDERLINE = zcommons.colorama.Style.DOUBLE_UNDERLINE



import time
from time import *


# For python version < 3.7
if "time_ns" not in globals():
    __SCALE = 1000000000

    def time_ns():
        return round(__SCALE * time())

    def process_time_ns():
        return round(__SCALE * process_time())

    def perf_counter_ns():
        return round(__SCALE * perf_counter())


def second_time():
    return int(time.time())


def milli_time():
    return int(1000 * time.time())
